#####
###  PHParser - a PHP-Parser for ZOPE
##   Copyright (C) 2002	Maik Jablonski (maik.jablonski@uni-bielefeld.de)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
##   along with this program; if not, write to the Free Software
###  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#####

# Path to the PHP-CGI-Interpreter

PHPath="php" 		# Linux & other *ix
# PHPath="c:\php\php"	# Windows

PHPEOL   = "\r\n"    # This marks usually the end of a PHP-Header-Line.
# PHPEOL = "\n"	     # Maybe this will work if you have problems with headers.

from Globals import HTML, DTMLFile, InitializeClass
from OFS.content_types import guess_content_type
from OFS.DTMLMethod import decapitate
from OFS.DTMLDocument import DTMLDocument
from AccessControl import getSecurityManager
from urllib import quote
from os import putenv,popen2
from urllib import urlencode
import cgi, re

_marker=[]

default_dd_html="""
<dtml-var standard_html_header>

<?php
	echo "<h2><dtml-var title_or_id> / $PHP_SELF</h2>";
	phpinfo();
?>

<dtml-var standard_html_footer>
"""

addPHPForm = DTMLFile('dtml/addPHPForm', globals())

# This code is taken and slightly modified from OFS/DTMLDocument.py

def addPHParser(self, id, title='', file='', REQUEST=None, submit=None):
    """Add a PHParser object with the contents of file. If
    'file' is empty, default document text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file: file=default_dd_html
    id=str(id)
    title=str(title)
    ob=PHParser(file, __name__=id)
    ob.title=title
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''
       

class PHParser(DTMLDocument):
    """ PHParser: It's a DTMLDocument which will be parsed by PHP after
        rendering the DTML """    
       
    meta_type = 'PHParser'
    manage_options=DTMLDocument.manage_options


    # This is the function where the document is piped through PHP

    def __phparse__(self,src,REQUEST={}):
         """ Pipes the rendered DTML through PHP """

         FORM_ITEMS  = REQUEST.form.items()
	 
	 # Parse QUERY_STRING into list...
	 
	 QUERY_ITEMS = cgi.parse_qsl(REQUEST.environ.get('QUERY_STRING',''))
	 
	 # Make a union of FORM_ITEMS & QUERY_ITEMS,
	 # so we avoid doubled items...
	 
         union=list(FORM_ITEMS)

	 for item in QUERY_ITEMS:
	     if not item in union:
	        union.append(item)
   
         # Put the QUERY_STRING into the environment
	 
         putenv('QUERY_STRING',urlencode(union))

	 # Ladies and gentlemen... and now... PHParsing...
	  
         (stin,stout)=popen2(PHPath)
         stin.write('<?php $PHP_SELF="%s"; ?>' % (self.absolute_url()))
         stin.write(src)
         stin.close()
         raw=stout.read()	
         stout.close()

	 # Now decode the Header and Body of the result
	 
	 (phpHeader,phpBody) = re.search("(?s)(?P<header>^.*?)%s(?P<body>.*)" % (PHPEOL*2),raw).groups() 
	 for line in phpHeader.split(PHPEOL):
	     if ':' in line:
	        (name,value) = line.split(':',1)
	        if name.lower()=='location':
	     	   REQUEST.RESPONSE.redirect(value.strip())
                else:
	           REQUEST.RESPONSE.setHeader(name,value.strip())
	 return(phpBody)

	 
    # This is the original code from OFS/DTMLDocument.py
    # Only a call to __phparse__ is added (see below)

    def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
        """Render the document given a client object, REQUEST mapping,
        Response, and key word arguments."""

        if not self._cache_namespace_keys:
            data = self.ZCacheable_get(default=_marker)
            if data is not _marker:
                # Return cached results.
                return data

        kw['document_id']   =self.getId()
        kw['document_title']=self.title
        if hasattr(self, 'aq_explicit'):
            bself=self.aq_explicit
        else: bself=self
        
        security=getSecurityManager()
        security.addContext(self)

        try:
            if client is None:
                # Called as subtemplate, so don't need error propigation!
                r=apply(HTML.__call__, (self, bself, REQUEST), kw)
		r=self.__phparse__(r,bself.REQUEST)
                if RESPONSE is None: result = r
                else: result = decapitate(r, RESPONSE)
                if not self._cache_namespace_keys:
                    self.ZCacheable_set(result)
                return result

            r=apply(HTML.__call__, (self, (client, bself), REQUEST), kw)
	    r=self.__phparse__(r,bself.REQUEST)
	    
            if type(r) is not type('') or RESPONSE is None:
                if not self._cache_namespace_keys:
                    self.ZCacheable_set(r)
                return r

        finally: security.removeContext(self)
	
        have_key=RESPONSE.headers.has_key
        if not (have_key('content-type') or have_key('Content-Type')):
            if self.__dict__.has_key('content_type'):
                c=self.content_type
            else:
                c, e=guess_content_type(self.__name__, r)
            RESPONSE.setHeader('Content-Type', c)
        result = decapitate(r, RESPONSE)
        if not self._cache_namespace_keys:
            self.ZCacheable_set(result)
        return result

InitializeClass(PHParser)
